<?php
/**
 * Donation Total
 *
 * @package     AutomatorWP\Integrations\Give\Triggers\Donation_Total
 * @author      AutomatorWP <contact@automatorwp.com>, Ruben Garcia <rubengcdev@gmail.com>
 * @since       1.0.0
 */
// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

class AutomatorWP_Give_Donation_Total extends AutomatorWP_Integration_Trigger {

    public $integration = 'give';
    public $trigger = 'give_donation_total';

    /**
     * Register the trigger
     *
     * @since 1.0.0
     */
    public function register() {

        automatorwp_register_trigger( $this->trigger, array(
            'integration'       => $this->integration,
            'label'             => __( 'User performs a donation through a form with a total greater than, less than or equal to a specific amount', 'automatorwp-give' ),
            'select_option'     => __( 'User performs a donation through a form with a total <strong>greater than, less than or equal</strong> to a specific amount', 'automatorwp-give' ),
            /* translators: %1$s: Post title. %2$s: Condition. %3$s: Amount. %4$s: Number of times. */
            'edit_label'        => sprintf( __( 'User performs a donation through %1$s with a total %2$s %3$s %4$s time(s)', 'automatorwp-give' ), '{post}', '{condition}', '{amount}', '{times}' ),
            /* translators: %1$s: Post title. %2$s: Condition. %3$s: Amount. */
            'log_label'         => sprintf( __( 'User performs a donation through %1$s with a total %2$s %3$s', 'automatorwp-give' ), '{post}', '{condition}', '{amount}' ),
            'action'            => 'give_update_payment_status',
            'function'          => array( $this, 'listener' ),
            'priority'          => 10,
            'accepted_args'     => 3,
            'options'           => array(
                'post' => automatorwp_utilities_post_option( array(
                    'name' => __( 'Form:', 'automatorwp-give' ),
                    'option_none_label' => __( 'any from', 'automatorwp-give' ),
                    'post_type' => 'give_forms'
                ) ),
                'condition' => automatorwp_utilities_condition_option(),
                'amount' => array(
                    'from' => 'amount',
                    'fields' => array(
                        'amount' => array(
                            'name' => __( 'Amount:', 'automatorwp-give' ),
                            'desc' => __( 'The order total required. Support decimals.', 'automatorwp-give' ),
                            'type' => 'text',
                            'attributes' => array(
                                'type' => 'number',
                                'min' => '0',
                            ),
                            'default' => 0
                        )
                    )
                ),
                'times' => automatorwp_utilities_times_option(),
            ),
            'tags' => array_merge(
                automatorwp_utilities_times_tag()
            )
        ) );

    }

    /**
     * Trigger listener
     *
     * @since 1.0.0
     *
     * @param int       $payment_id     The payment ID.
     * @param string    $new_status     The payment new status.
     * @param string    $old_status     The payment old status.
     */
    public function listener( $payment_id, $new_status, $old_status ) {

        // Make sure that payments are only completed once.
        if ( $old_status === 'publish' || $old_status === 'complete' ) {
            return;
        }

        // Make sure the payment completion is only processed when new status is complete.
        if ( $new_status !== 'publish' && $new_status !== 'complete' ) {
            return;
        }

        // Setup the payment data
        $payment        = new Give_Payment( $payment_id );
        $user_id        = $payment->user_id;
        $donor_id       = $payment->customer_id;
        $total          = $payment->total;
        $form_id        = $payment->form_id;

        // Trigger the donation total
        automatorwp_trigger_event( array(
            'trigger'           => $this->trigger,
            'user_id'           => $donor_id,
            'post_id'           => $form_id,
            'donation_total'    => $total,
        ) );

    }

    /**
     * User deserves check
     *
     * @since 1.0.0
     *
     * @param bool      $deserves_trigger   True if user deserves trigger, false otherwise
     * @param stdClass  $trigger            The trigger object
     * @param int       $user_id            The user ID
     * @param array     $event              Event information
     * @param array     $trigger_options    The trigger's stored options
     * @param stdClass  $automation         The trigger's automation object
     *
     * @return bool                          True if user deserves trigger, false otherwise
     */
    public function user_deserves_trigger( $deserves_trigger, $trigger, $user_id, $event, $trigger_options, $automation ) {

        // Don't deserve if post and donation total are not received
        if( ! isset( $event['post_id'] ) && ! isset( $event['donation_total'] ) ) {
            return false;
        }

        // Don't deserve if post doesn't match with the trigger option
        if( ! automatorwp_posts_matches( $event['post_id'], $trigger_options['post'] ) ) {
            return false;
        }

        $total = $event['donation_total'];
        $required_total =  (float) $trigger_options['amount'];

        // Don't deserve if donation total doesn't match with the trigger option
        if( ! automatorwp_number_condition_matches( $total, $required_total, $trigger_options['condition'] ) ) {
            return false;
        }

        return $deserves_trigger;

    }

}

new AutomatorWP_Give_Donation_Total();