<?php
/**
 * Donation
 *
 * @package     AutomatorWP\Integrations\Give\Triggers\Donation
 * @author      AutomatorWP <contact@automatorwp.com>, Ruben Garcia <rubengcdev@gmail.com>
 * @since       1.0.0
 */
// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

class AutomatorWP_Give_Donation extends AutomatorWP_Integration_Trigger {

    public $integration = 'give';
    public $trigger = 'give_donation';

    /**
     * Register the trigger
     *
     * @since 1.0.0
     */
    public function register() {

        automatorwp_register_trigger( $this->trigger, array(
            'integration'       => $this->integration,
            'label'             => __( 'User performs a donation through a form', 'automatorwp-give' ),
            'select_option'     => __( 'User performs a <strong>donation</strong> through a form', 'automatorwp-give' ),
            /* translators: %1$s: Post title. %2$s: Number of times. */
            'edit_label'        => sprintf( __( 'User performs a donation through %1$s %2$s time(s)', 'automatorwp-give' ), '{post}', '{times}' ),
            /* translators: %1$s: Post title. */
            'log_label'         => sprintf( __( 'User performs a donation through %1$s', 'automatorwp-give' ), '{post}' ),
            'action'            => 'give_update_payment_status',
            'function'          => array( $this, 'listener' ),
            'priority'          => 10,
            'accepted_args'     => 3,
            'options'           => array(
                'post' => automatorwp_utilities_post_option( array(
                    'name' => __( 'Form:', 'automatorwp-give' ),
                    'option_none_label' => __( 'any from', 'automatorwp-give' ),
                    'post_type' => 'give_forms'
                ) ),
                'times' => automatorwp_utilities_times_option(),
            ),
            'tags' => array_merge(
                automatorwp_utilities_post_tags(),
                automatorwp_utilities_times_tag()
            )
        ) );

    }

    /**
     * Trigger listener
     *
     * @since 1.0.0
     *
     * @param int       $payment_id     The payment ID.
     * @param string    $new_status     The payment new status.
     * @param string    $old_status     The payment old status.
     */
    public function listener( $payment_id, $new_status, $old_status ) {

        // Make sure that payments are only completed once.
        if ( $old_status === 'publish' || $old_status === 'complete' ) {
            return;
        }

        // Make sure the payment completion is only processed when new status is complete.
        if ( $new_status !== 'publish' && $new_status !== 'complete' ) {
            return;
        }

        // Setup the payment data
        $payment        = new Give_Payment( $payment_id );
        $user_id        = $payment->user_id;
        $donor_id       = $payment->customer_id;
        $total          = $payment->total;
        $form_id        = $payment->form_id;

        // Trigger the donation
        automatorwp_trigger_event( array(
            'trigger'           => $this->trigger,
            'user_id'           => $donor_id,
            'post_id'           => $form_id,
            'donation_total'    => $total,
        ) );

    }

    /**
     * User deserves check
     *
     * @since 1.0.0
     *
     * @param bool      $deserves_trigger   True if user deserves trigger, false otherwise
     * @param stdClass  $trigger            The trigger object
     * @param int       $user_id            The user ID
     * @param array     $event              Event information
     * @param array     $trigger_options    The trigger's stored options
     * @param stdClass  $automation         The trigger's automation object
     *
     * @return bool                          True if user deserves trigger, false otherwise
     */
    public function user_deserves_trigger( $deserves_trigger, $trigger, $user_id, $event, $trigger_options, $automation ) {

        // Don't deserve if post is not received
        if( ! isset( $event['post_id'] ) ) {
            return false;
        }

        // Don't deserve if post doesn't match with the trigger option
        if( ! automatorwp_posts_matches( $event['post_id'], $trigger_options['post'] ) ) {
            return false;
        }

        return $deserves_trigger;

    }

}

new AutomatorWP_Give_Donation();