<?php
/**
 * Generate Hash
 *
 * @package     AutomatorWP\Integrations\Generator\Actions\Generate_Hash
 * @author      AutomatorWP <contact@automatorwp.com>, Ruben Garcia <rubengcdev@gmail.com>
 * @since       1.0.0
 */
// Exit if accessed directly

if( !defined( 'ABSPATH' ) ) exit;

class AutomatorWP_Generator_Generate_Hash extends AutomatorWP_Integration_Action {

    public $integration = 'generator';
    public $action = 'generator_generate_hash';

    /**
     * Register the trigger
     *
     * @since 1.0.0
     */
    public function register() {

        automatorwp_register_action( $this->action, array(
            'integration'       => $this->integration,
            'label'             => __( 'Generate a hash', 'automatorwp' ),
            'select_option'     => __( 'Generate a <strong>hash</strong>', 'automatorwp' ),
            /* translators: %1$s: Hash algorithm. */
            'edit_label'        => sprintf( __( 'Generate a hash with %1$s', 'automatorwp' ), '{random_hash}' ),
            /* translators: %1$s: Hash algorithm. */
            'log_label'         => sprintf( __( 'Generate a hash with %1$s', 'automatorwp' ), '{random_hash}' ),
            'options'           => array(
                'random_hash' => array(
                    'from'      => 'algorithm',
                    'default'   => __( 'algorithm', 'automatorwp' ),
                    'fields' => array(
                        'data' => array(
                            'name' => __( 'Data:', 'automatorwp' ),
                            'desc' => __( 'Data to be hashed.', 'automatorwp' ),
                            'type' => 'text',
                            'default' => ''
                        ),
                        'algorithm' => automatorwp_utilities_ajax_selector_field( array(
                            'option_none' => false,
                            'option_custom' => false,
                            'placeholder'       => __( 'Select an algorithm', 'automatorwp' ),
                            'name'              => __( 'Algorithm:', 'automatorwp' ),
                            'action_cb'         => 'automatorwp_generator_get_algorithms',
                            'options_cb'        => 'automatorwp_generator_options_cb_algorithm',
                            'default'           => ''
                        ) ),
                     ) )
            ),
            'tags'  => automatorwp_generator_get_actions_random_hash_tags()
        ) );

    }

    /**
     * Action execution function
     *
     * @since 1.0.0
     *
     * @param stdClass  $action             The action object
     * @param int       $user_id            The user ID
     * @param array     $action_options     The action's stored options (with tags already passed)
     * @param stdClass  $automation         The action's automation object
     */
    public function execute( $action, $user_id, $action_options, $automation ) {

        $this->result = '';
        
        // Shorthand
        $data = $action_options['data'];
        $algorithm = $action_options['algorithm'];
        
        // Bail if not data
        if ( empty( $data ) ) {
            $this->result = __( 'Please, insert data to be hashed', 'automatorwp' );
            return;
        }
        
        // Bail if not algorithm
        if ( empty( $algorithm ) ) {
            $this->result = __( 'Please, select an algorithm', 'automatorwp' );
            return;
        }

        // Get the algorithms
        $all_algorithms = hash_algos();

        $this->random_hash = hash( $all_algorithms[$algorithm], $data ); 
        
        $this->result = sprintf( __( '%s', 'automatorwp' ), $this->random_hash );

    }

    /**
     * Register required hooks
     *
     * @since 1.0.0
     */
    public function hooks() {

        // Log meta data
        add_filter( 'automatorwp_user_completed_action_log_meta', array( $this, 'log_meta' ), 10, 5 );

        // Log fields
        add_filter( 'automatorwp_log_fields', array( $this, 'log_fields' ), 10, 5 );

        parent::hooks();

    }

    /**
     * Action custom log meta
     *
     * @since 1.0.0
     *
     * @param array     $log_meta           Log meta data
     * @param stdClass  $action             The action object
     * @param int       $user_id            The user ID
     * @param array     $action_options     The action's stored options (with tags already passed)
     * @param stdClass  $automation         The action's automation object
     *
     * @return array
     */
    public function log_meta( $log_meta, $action, $user_id, $action_options, $automation ) {

        // Bail if action type don't match this action
        if( $action->type !== $this->action ) {
            return $log_meta;
        }

        // Store the action's result
        $log_meta['result'] = $this->result;
        $log_meta['random_hash'] = ( isset( $this->random_hash ) ? $this->random_hash : '' );

        return $log_meta;
    }

    /**
     * Action custom log fields
     *
     * @since 1.0.0
     *
     * @param array     $log_fields The log fields
     * @param stdClass  $log        The log object
     * @param stdClass  $object     The trigger/action/automation object attached to the log
     *
     * @return array
     */
    public function log_fields( $log_fields, $log, $object ) {

        // Bail if log is not assigned to an action
        if( $log->type !== 'action' ) {
            return $log_fields;
        }

        // Bail if action type don't match this action
        if( $object->type !== $this->action ) {
            return $log_fields;
        }

        $log_fields['result'] = array(
            'name' => __( 'Result:', 'automatorwp' ),
            'type' => 'text',
        );

        return $log_fields;
    }

}

new AutomatorWP_Generator_Generate_Hash();